//
// uncia - a big cat
// Copyright (C) 2008, 2010 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libuncia/input/filter.h>


libuncia::input_filter::~input_filter()
{
    delete [] getline_buf;
}


libuncia::input_filter::input_filter(const pointer &a_deeper) :
    deeper(a_deeper),
    getline_buf(0),
    getline_max(0)
{
}


std::string
libuncia::input_filter::filename_get()
    const
{
    return deeper.filename_get();
}


ssize_t
libuncia::input_filter::deeper_read(void *data, size_t size)
{
    char *start = (char *)data;
    char *end = start + size;
    char *cp = start;
    while (cp < end)
    {
        if (!holding_pen.empty())
        {
            *cp++ = holding_pen.back();
            holding_pen.pop_back();
            continue;
        }

        //
        // Grab a character.  This is necessary to prime the buffer for
        // the deeper.readsome call.
        //
        if (!deeper.get(*cp))
            break;
        ++cp;

        // slurp the rest of the deeper buffer, if we can
        size_t maxlen = end - cp;
        if (maxlen)
        {
            ssize_t n = deeper.readsome(cp, maxlen);
            if (n > 0)
            {
                cp += n;
                break;
            }
        }
    }
    return (cp - start);
}


void
libuncia::input_filter::deeper_putback(const std::string &text)
{
    const char *start = text.c_str();
    const char *cp = start + text.size();
    while (cp > start)
    {
        --cp;
        deeper_putback(*cp);
    }
}


#include <cstring>


bool
libuncia::input_filter::deeper_getline(std::string &result)
{
    size_t bufsiz = 0;
    for (;;)
    {
        char c = 0;
        if (!deeper_get(c))
        {
            if (bufsiz == 0)
                return false;
            c = '\n';
        }
        if (c == '\n')
        {
            result = std::string(getline_buf, bufsiz);
            return true;
        }

        if (bufsiz >= getline_max)
        {
            size_t new_getline_max = getline_max * 2 + 64;
            char *new_buffer = new char [new_getline_max];
            memcpy(new_buffer, getline_buf, bufsiz);
            delete [] getline_buf;
            getline_buf = new_buffer;
            getline_max = new_getline_max;
        }
        getline_buf[bufsiz++] = c;
    }
}
