//
// uncia - a big cat
// Copyright (C) 2010 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libuncia/input/filter/ascii85.h>


libuncia::input_filter_ascii85::~input_filter_ascii85()
{
}


libuncia::input_filter_ascii85::input_filter_ascii85(const pointer &a_deeper) :
    input_filter(a_deeper),
    transparent(true),
    accumulator(0),
    accumulator_nbytes(0)
{
}


libuncia::input::pointer
libuncia::input_filter_ascii85::create(const pointer &a_deeper)
{
    return pointer(new input_filter_ascii85(a_deeper));
}


bool
libuncia::input_filter_ascii85::get1(unsigned char &buf)
{
    for (;;)
    {
        char c = 0;
        if (!deeper_get(c))
        {
            return false;
        }
        switch (c)
        {
        default:
            deeper_putback(c);
            return false;

        case '\0':
        case '\t':
        case '\n':
        case '\f':
        case '\r':
        case ' ':
        case '\b':
        case 0177:
            break;

        case '!':
        case '"':
        case '#':
        case '$':
        case '%':
        case '&':
        case '\'':
        case '(':
        case ')':
        case '*':
        case '+':
        case ',':
        case '-':
        case '.':
        case '/':
        case '0':
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':
        case '9':
        case ':':
        case ';':
        case '<':
        case '=':
        case '>':
        case '?':
        case '@':
        case 'A':
        case 'B':
        case 'C':
        case 'D':
        case 'E':
        case 'F':
        case 'G':
        case 'H':
        case 'I':
        case 'J':
        case 'K':
        case 'L':
        case 'M':
        case 'N':
        case 'O':
        case 'P':
        case 'Q':
        case 'R':
        case 'S':
        case 'T':
        case 'U':
        case 'V':
        case 'W':
        case 'X':
        case 'Y':
        case 'Z':
        case '[':
        case '\\':
        case ']':
        case '^':
        case '_':
        case '`':
        case 'a':
        case 'b':
        case 'c':
        case 'd':
        case 'e':
        case 'f':
        case 'g':
        case 'h':
        case 'i':
        case 'j':
        case 'k':
        case 'l':
        case 'm':
        case 'n':
        case 'o':
        case 'p':
        case 'q':
        case 'r':
        case 's':
        case 't':
        case 'u':
            buf = c - '!';
            return true;
        }
    }
}


bool
libuncia::input_filter_ascii85::get5(unsigned long &buf, unsigned &nbytes)
{
    unsigned char c1 = 0;
    if (!get1(c1))
        return false;

    unsigned char c2 = 0;
    if (!get1(c2))
    {
        deeper_putback(c1 + '!');
        return false;
    }

    nbytes = 1;
    unsigned char c3 = 0;
    unsigned char c4 = 0;
    unsigned char c5 = 0;
    if (get1(c3))
    {
        ++nbytes;
        if (get1(c4))
        {
            ++nbytes;
            if (get1(c5))
                ++nbytes;
        }
    }

    buf = ((((c1 * 85) + c2) * 85 + c3) * 85 + c4) * 85 + c5;
    return true;
}


ssize_t
libuncia::input_filter_ascii85::read(void *data, size_t size)
{
    char *start = (char *)data;
    char *end = start + size;
    char *cp = start;
    while (cp < end)
    {
        if (transparent)
        {
            char c = 0;
            if (!deeper_get(c))
                break;
            if (c == '<')
            {
                char c2 = 0;
                if (deeper_get(c2))
                {
                    if (c2 == '~')
                    {
                        transparent = false;
                        continue;
                    }
                    deeper_putback(c2);
                }
            }
            if (!isspace((unsigned char)c))
                invalid_format();
            continue;
        }

        switch (accumulator_nbytes)
        {
        case 1:
            *cp++ = accumulator;
            --accumulator_nbytes;
            continue;

        case 2:
            *cp++ = accumulator >> 8;
            --accumulator_nbytes;
            continue;

        case 3:
            *cp++ = accumulator >> 16;
            --accumulator_nbytes;
            continue;

        case 4:
            *cp++ = accumulator >> 24;
            --accumulator_nbytes;
            continue;
        }

        char c = 0;
        if (!deeper_get(c))
            break;

        switch (c)
        {
        case '~':
            {
                char c2 = 0;
                if (deeper_get(c2))
                {
                    if (c2 == '>')
                    {
                        transparent = true;
                        continue;
                    }
                }
                invalid_format();
                //NOTREACHED
                continue;
            }

        case 'z':
            accumulator = 0;
            accumulator_nbytes = 4;
            continue;

        case 'y':
            accumulator = 0x20202020;
            accumulator_nbytes = 4;
            continue;
        }

        deeper_putback(c);
        if (!get5(accumulator, accumulator_nbytes))
            invalid_format();
        switch (accumulator_nbytes)
        {
        case 1:
            // 0xKK000000 -> 0x000000KK
            accumulator >>= 24;
            break;
            break;

        case 2:
            // 0xKKKK0000 -> 0x0000KKKK
            accumulator >>= 16;
            break;

        case 3:
            // 0xKKKKKK00 -> 0x00KKKKKK
            accumulator >>= 8;
            break;
        }
    }
    return (cp - start);
}


void
libuncia::input_filter_ascii85::invalid_format(void)
{
    fatal_error("not correct ascii-85 format");
}
