//
// uncia - a big cat
// Copyright (C) 2010 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <cstdio> // for sscanf

#include <libuncia/input/filter/base85.h>


libuncia::input_filter_base85::~input_filter_base85()
{
}


libuncia::input_filter_base85::input_filter_base85(const pointer &a_deeper) :
    input_filter(a_deeper),
    state(state_initial),
    acc(0),
    nbytes(0),
    check_xor(0),
    check_sum(0),
    check_rot(0),
    check_tot(0)
{
}


libuncia::input::pointer
libuncia::input_filter_base85::create(const pointer &a_deeper)
{
    return pointer(new input_filter_base85(a_deeper));
}


bool
libuncia::input_filter_base85::get1(unsigned char &buf)
{
    for (;;)
    {
        char c = 0;
        if (!deeper_get(c))
        {
            return false;
        }
        switch (c)
        {
        default:
            deeper_putback(c);
            return false;

        case '\0':
        case '\t':
        case '\n':
        case '\f':
        case '\r':
        case ' ':
        case '\b':
        case 0177:
            break;

        case '!':
        case '"':
        case '#':
        case '$':
        case '%':
        case '&':
        case '\'':
        case '(':
        case ')':
        case '*':
        case '+':
        case ',':
        case '-':
        case '.':
        case '/':
        case '0':
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':
        case '9':
        case ':':
        case ';':
        case '<':
        case '=':
        case '>':
        case '?':
        case '@':
        case 'A':
        case 'B':
        case 'C':
        case 'D':
        case 'E':
        case 'F':
        case 'G':
        case 'H':
        case 'I':
        case 'J':
        case 'K':
        case 'L':
        case 'M':
        case 'N':
        case 'O':
        case 'P':
        case 'Q':
        case 'R':
        case 'S':
        case 'T':
        case 'U':
        case 'V':
        case 'W':
        case 'X':
        case 'Y':
        case 'Z':
        case '[':
        case '\\':
        case ']':
        case '^':
        case '_':
        case '`':
        case 'a':
        case 'b':
        case 'c':
        case 'd':
        case 'e':
        case 'f':
        case 'g':
        case 'h':
        case 'i':
        case 'j':
        case 'k':
        case 'l':
        case 'm':
        case 'n':
        case 'o':
        case 'p':
        case 'q':
        case 'r':
        case 's':
        case 't':
        case 'u':
            buf = c - '!';
            return true;
        }
    }
}


bool
libuncia::input_filter_base85::get5(unsigned long &buf)
{
    unsigned char c1 = 0;
    if (!get1(c1))
        return false;
    unsigned char c2 = 0;
    if (!get1(c2))
        return false;
    unsigned char c3 = 0;
    if (!get1(c3))
        return false;
    unsigned char c4 = 0;
    if (!get1(c4))
        return false;
    unsigned char c5 = 0;
    if (!get1(c5))
        return false;
    buf = ((((c1 * 85) + c2) * 85 + c3) * 85 + c4) * 85 + c5;
    return true;
}


void
libuncia::input_filter_base85::check_for_length(void)
{
    for (;;)
    {
        char c = 0;
        if (!deeper_get(c))
            return;
        switch (c)
        {
        case 'x':
            {
                std::string line;
                deeper_getline(line);

                deeper_putback('\n');
                deeper_putback(line.c_str());
                deeper_putback('x');

                long n1 = 0;
                if (sscanf(line.c_str(), "btoa End N %ld ", &n1) != 1)
                {
                    format_error();
                }
                n1 &= 3;
                if (n1)
                    nbytes = n1;
                return;
            }

        case '\0':
        case '\t':
        case '\n':
        case '\f':
        case '\r':
        case ' ':
        case '\b':
        case 0177:
            continue;

        default:
            deeper_putback(c);
            return;
        }
        break;
    }
}


void
libuncia::input_filter_base85::check_last_line(void)
{
    std::string line;
    if (!deeper_getline(line))
    {
        format_error();
    }

    unsigned long n1 = 0;
    unsigned long n2 = 0;
    unsigned long n3 = 0;
    unsigned long n4 = 0;
    unsigned long n5 = 0;
    int n =
        sscanf
        (
            line.c_str(),
            "btoa End N %ld %lx E %lx S %lx R %lx",
            &n1,
            &n2,
            &n3,
            &n4,
            &n5
        );
    if (n != 5 || n1 != n2 || n1 != check_tot)
        format_error();
    while (check_tot & 3)
        sum(0);
    if (check_xor != n3 || check_sum != n4 || check_rot != n5)
        format_error();
    // FIXME: check the checksums
}


ssize_t
libuncia::input_filter_base85::read(void *data, size_t size)
{
    switch (state)
    {
    case state_initial:
        for (;;)
        {
            std::string line;
            if (!deeper_getline(line))
                format_error();
            if (line == "xbtoa Begin")
            {
                state = state_decode;
                break;
            }
        }
        break;

    case state_decode:
        break;

    case state_eof:
        return 0;
    }

    char *start = (char *)data;
    char *end = start + size;
    char *cp = start;
    while (cp < end)
    {
        if (nbytes > 0)
        {
            unsigned char c = acc >> 24;
            acc <<= 8;
            --nbytes;
            *cp++ = c;
            sum(c);
            continue;
        }

        char c = 0;
        if (!deeper_get(c))
            break;
        switch (c)
        {
        case '\t':
        case '\n':
        case '\f':
        case '\r':
        case ' ':
            continue;

        case 'x':
            check_last_line();
            while (deeper_get(c))
                ;
            state = state_eof;
            continue;

        case 'y':
            acc = 0x20202020;
            nbytes = 4;
            break;

        case 'z':
            acc = 0;
            nbytes = 4;
            break;

        default:
            deeper_putback(c);
            if (!get5(acc))
                format_error();
            nbytes = 4;
            break;
        }
        check_for_length();
    }
    return (cp - start);
}


void
libuncia::input_filter_base85::format_error(void)
{
    fatal_error("does not look like a valid base-85 (btoa) stream");
}


void
libuncia::input_filter_base85::sum(unsigned char c)
{
    check_xor ^= c;
    check_sum = (check_sum + c + 1) & 0xFFFFFFFF;

    check_rot = (check_rot << 1) | ((check_rot >> 31) & 1);
    check_rot = (check_rot + c) & 0xFFFFFFFF;

    check_tot = (check_tot + 1) & 0xFFFFFFFF;
}
