//
// uncia - a big cat
// Copyright (C) 2010 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_INPUT_FILTER_BASE85_H
#define LIBUNCIA_INPUT_FILTER_BASE85_H

#include <libuncia/input/filter.h>
#include <libuncia/istream.h>

namespace libuncia
{
    /**
      * The input_filter_base85 class is used to represent an input filter
      * which decodes the base 85 encoding (like the atob(1) command).
      * See http://en.wikipedia.org/wiki/Ascii85 for more information.
      */
    class input_filter_base85:
        public input_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~input_filter_base85();

        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     where to obtain the input to this filter.
          */
        static pointer create(const pointer &deeper);

    protected:
        // See base class for documentation.
        ssize_t read(void *data, size_t size);

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     where to obtain the input to this filter.
          */
        input_filter_base85(const pointer &deeper);

        enum state_t
        {
            state_initial,
            state_decode,
            state_eof
        };

        state_t state;

        unsigned long acc;
        unsigned nbytes;

        bool get1(unsigned char &buf);
        bool get5(unsigned long &buf);

        void check_for_length(void);
        void check_last_line(void);
        void format_error(void) NORETURN;

        unsigned long check_xor;
        unsigned long check_sum;
        unsigned long check_rot;
        unsigned long check_tot;

        void sum(unsigned char c);

        /**
          * The default constructor.  Do not use.
          */
        input_filter_base85();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        input_filter_base85(const input_filter_base85 &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        input_filter_base85 &operator=(const input_filter_base85 &rhs);
    };
}

#endif // LIBUNCIA_INPUT_FILTER_BASE85_H
