//
// uncia - a big cat
// Copyright (C) 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libuncia/input/filter/vms.h>


libuncia::input_filter_vms::~input_filter_vms()
{
}


libuncia::input_filter_vms::input_filter_vms(const pointer &a_deeper) :
    input_filter(a_deeper),
    address(0),
    state(state_expecting_len1),
    length(0)
{
}


libuncia::input::pointer
libuncia::input_filter_vms::create(const pointer &a_deeper)
{
    return pointer(new input_filter_vms(a_deeper));
}


unsigned char
libuncia::input_filter_vms::getch(void)
{
    char c = 0;
    if (!deeper_get(c))
        fatal_error("not in VMS format");
    ++address;
    return c;
}


ssize_t
libuncia::input_filter_vms::read(void *data, size_t size)
{
    char *start = (char *)data;
    char *end = start + size;
    char *cp = start;
    while (cp < end)
    {
        switch (state)
        {
        case state_eof:
            break;

        case state_expecting_len1:
            if (address & 1)
                getch();
            length = getch();
            state = state_expecting_len2;
            continue;

        case state_expecting_len2:
            length |= getch() << 8;
            if (length == 0xFFFF)
            {
                state = state_eof;

                // discard the rest of the inpout
                char c = 0;
                while (deeper_get(c))
                    ;
            }
            else if (length == 0x0000)
                state = state_eoln;
            else
                state = state_body;
            continue;

        case state_body:
            *cp++ = getch();
            --length;
            if (length == 0)
                state = state_eoln;
            continue;

        case state_eoln:
            *cp++ = '\n';
            state = state_expecting_len1;
            break;
        }
        break;
    }
    return (cp - start);
}


// vim: set ts=8 sw=4 et :
