//
// uncia - a big cat
// Copyright (C) 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_INPUT_FILTER_VMS_H
#define LIBUNCIA_INPUT_FILTER_VMS_H

#include <libuncia/input/filter.h>

namespace libuncia
{
    /**
      * The input_filter_vms class is used to represent an input filter
      * which converts a VMS text file into normalised input (lines
      * terminated with '\n').
      */
    class input_filter_vms:
        public input_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~input_filter_vms();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     where to obtain the input to this filter.
          */
        input_filter_vms(const pointer &deeper);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     where to obtain the input to this filter.
          */
        static pointer create(const pointer &deeper);

    protected:
        // See base class for documentation.
        ssize_t read(void *data, size_t size);

    private:
        /**
          * The address instance variable is used to remember the
          * position within the file, because there may be a padding
          * byte before the line length, to ensure the length is always
          * word aligned.
          */
        unsigned long address;

        enum state_t
        {
            state_expecting_len1,
            state_expecting_len2,
            state_body,
            state_eoln,
            state_eof,
        };

        /**
          * The state instance variable is used to remember where we are
          * up to when parsing each line of the file.
          */
        state_t state;

        /**
          * The length instance variable is used to remember how much of
          * the line is left, when in state_data.
          */
        unsigned short length;

        /**
          * The getch method is used to obtain the next character of the
          * input, there should always be one, because VMS text files
          * have an explicit EOF indicator.
          */
        unsigned char getch(void);

        /**
          * The default constructor.  Do not use.
          */
        input_filter_vms();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        input_filter_vms(const input_filter_vms &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        input_filter_vms &operator=(const input_filter_vms &rhs);
    };
}

// vim: set ts=8 sw=4 et :
#endif // LIBUNCIA_INPUT_FILTER_VMS_H
