//
//      uncia - a big cat
//      Copyright (C) 2008, 2010 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <libuncia/config.h>
#include <fcntl.h>
#include <libexplain/creat.h>
#include <libexplain/close.h>
#include <libexplain/write.h>

#include <libuncia/error_handler.h>
#include <libuncia/output/file.h>
#include <libuncia/output/null.h>
#include <libuncia/output/stdout.h>


libuncia::output_file::~output_file()
{
    if (close(fd) < 0)
    {
        error_handler_get()->fatal_error("%s", explain_close(fd));
    }
    fd = -1;
}


libuncia::output_file::output_file(const std::string &fn) :
    filename(fn),
    fd(-1)
{
    fd = creat(filename.c_str(), 0666);
    if (fd < 0)
    {
        error_handler_get()->fatal_error
        (
            "%s",
            explain_creat(filename.c_str(), 0666)
        );
    }
}


libuncia::output_file::pointer
libuncia::output_file::create(const std::string &fn)
{
    if
    (
        fn == "-"
    ||
        fn == "/dev/stdout"
    ||
        fn == "/dev/fd/1"
    ||
        fn == "/proc/self/fd/1"
    )
        return libuncia::output_stdout::create();
    if (fn == "/dev/null" || fn == "/dev/zero")
        return libuncia::output_null::create();
    return pointer(new output_file(fn));
}


ssize_t
libuncia::output_file::write(const void *data, size_t size)
{
    ssize_t ret = ::write(fd, data, size);
    if (ret < 0)
    {
        error_handler_get()->fatal_error("%s", explain_write(fd, data, size));
    }
    if ((size_t)ret != size)
    {
        error_handler_get()->fatal_error
        (
            "write %s: too short, asked to write %ld, but only wrote %ld",
            filename.c_str(),
            (long)size,
            (long)ret
        );
    }
    return ret;
}


int
libuncia::output_file::sync()
{
    // this means fflush, not fsync
    return 0;
}


std::string
libuncia::output_file::filename_get()
    const
{
    return filename;
}
