//
// uncia - a big cat
// Copyright (C) 2010 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libuncia/output/filter/ascii85.h>

#define MAXPERLINE 78


libuncia::output_filter_ascii85::~output_filter_ascii85()
{
    sync();
    if (accumulator_nbytes > 0)
        word_out(accumulator, accumulator_nbytes);
    if (column > MAXPERLINE - 2)
        deeper_putc('\n');
    deeper_puts("~>\n");
}


libuncia::output_filter_ascii85::output_filter_ascii85(
    const pointer &a_deeper
) :
    output_filter(a_deeper),
    accumulator(0),
    accumulator_nbytes(0),
    column(2)
{
    deeper_puts("<~");
}


libuncia::output::pointer
libuncia::output_filter_ascii85::create(const pointer &a_deeper)
{
    return pointer(new output_filter_ascii85(a_deeper));
}


void
libuncia::output_filter_ascii85::char_out(unsigned char c)
{
    deeper_putc(c);
    ++column;
    if (column >= MAXPERLINE)
    {
        deeper_putc('\n');
        column = 0;
    }
}


void
libuncia::output_filter_ascii85::word_out(unsigned long word, unsigned len)
{
    switch (len)
    {
    case 1:
        {
            unsigned long x = word;
            x /= 85;
            x /= 85;
            x /= 85;
            unsigned char c2 = x % 85;
            x /= 85;
            assert(x < 83);
            unsigned char c1 = x;

            if (((((c1 * 85uL) + c2) * 85 * 85 * 85) & 0xFF000000) < word)
                ++c2;
            assert(((((c1 * 85uL) + c2) * 85 * 85 * 85) & 0xFF000000) == word);

            char_out('!' + c1);
            char_out('!' + c2);
        }
        break;

    case 2:
        {
            unsigned long x = word;
            x /= 85;
            x /= 85;
            unsigned char c3 = x % 85;
            x /= 85;
            unsigned char c2 = x % 85;
            x /= 85;
            assert(x < 83);
            unsigned char c1 = x;

            if
            (
                (((((c1 * 85uL) + c2) * 85 + c3) * 85 * 85) & 0xFFFF0000)
            <
                word
            )
            {
                ++c3;
            }
            assert((word & 0x0000FFFF) == 0);
            assert
            (
                (((((c1 * 85uL) + c2) * 85 + c3) * 85 * 85) & 0xFFFF0000)
            ==
                word
            );

            char_out('!' + c1);
            char_out('!' + c2);
            char_out('!' + c3);
        }
        break;

    case 3:
        {
            unsigned long x = word;
            x /= 85;
            unsigned char c4 = x % 85;
            x /= 85;
            unsigned char c3 = x % 85;
            x /= 85;
            unsigned char c2 = x % 85;
            x /= 85;
            assert(x < 83);
            unsigned char c1 = x;

            if
            (
                ((((((c1 * 85uL) + c2) * 85 + c3) * 85 + c4) * 85) & 0xFFFFFF00)
            <
                word
            )
                ++c4;
            assert
            (
                ((((((c1 * 85uL) + c2) * 85 + c3) * 85 + c4) * 85) & 0xFFFFFF00)
            ==
                word
            );

            char_out('!' + c1);
            char_out('!' + c2);
            char_out('!' + c3);
            char_out('!' + c4);
        }
        break;

    case 4:
    default:
        if (word == 0)
        {
            char_out('z');
        }
#if 0
        else if (word == 0x20202020)
        {
            // If we are being conservative, we should not use this code.
            char_out('y');
        }
#endif
        else
        {
            unsigned long x = word;
            unsigned char c5 = x % 85;
            x /= 85;
            unsigned char c4 = x % 85;
            x /= 85;
            unsigned char c3 = x % 85;
            x /= 85;
            unsigned char c2 = x % 85;
            x /= 85;
            assert(x < 83);
            unsigned char c1 = x;

            char_out('!' + c1);
            char_out('!' + c2);
            char_out('!' + c3);
            char_out('!' + c4);
            char_out('!' + c5);
        }
        break;
    }
}


void
libuncia::output_filter_ascii85::encode(unsigned char c)
{
    accumulator |= c << ((3 - accumulator_nbytes) * 8);
    ++accumulator_nbytes;
    if (accumulator_nbytes >= 4)
    {
        word_out(accumulator, accumulator_nbytes);
        accumulator = 0;
        accumulator_nbytes = 0;
    }
}


ssize_t
libuncia::output_filter_ascii85::write(const void *data, size_t size)
{
    const char *start = (const char *)data;
    const char *end = start + size;
    const char *cp = start;
    while (cp < end)
    {
        unsigned char c = *cp++;
        encode(c);
    }
    return size;
}
