//
// uncia - a big cat
// Copyright (C) 2010 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libuncia/output/filter/base32hex.h>


static char base32hex[] = "0123456789ABCDEFGHIJKLMNOPQRSTUV";


static inline char
map32(int n)
{
    return base32hex[n & 0x1F];
}


libuncia::output_filter_base32hex::~output_filter_base32hex()
{
    // The sequence goes like this:
    // first byte:  1 digit,  3 residual (1)
    // second byte: 2 digits, 1 residual (3)
    // third byte:  1 digit,  4 residual (4)
    // fourth byte: 2 digits, 2 residual (6)
    // fifth byte:  2 digits, 0 resudual (8)

    if (residual_bits)
    {
        deeper_putc(map32(residual_value << (5 - residual_bits)));
        ++output_column;
        while (output_column & 7)
        {
            deeper_putc('=');
            ++output_column;
        }
    }
    if (output_column)
        deeper_putc('\n');
}


libuncia::output_filter_base32hex::output_filter_base32hex(
    const pointer &a_deeper
) :
    output_filter(a_deeper),
    residual_value(0),
    residual_bits(0),
    output_column(0)
{
}


libuncia::output::pointer
libuncia::output_filter_base32hex::create(const pointer &a_deeper)
{
    return pointer(new output_filter_base32hex(a_deeper));
}


ssize_t
libuncia::output_filter_base32hex::write(const void *data, size_t size)
{
    const unsigned char *start = (const unsigned char *)data;
    const unsigned char *end = start + size;
    const unsigned char *cp = start;
    while (cp < end)
    {
        unsigned char c = *cp++;

        residual_value = (residual_value << 8) | c;
        residual_bits += 8;
        for (;;)
        {
            residual_bits -= 5;
            deeper_putc(map32(residual_value >> residual_bits));
            ++output_column;
            if (residual_bits == 0 && output_column > 70)
            {
                deeper_putc('\n');
                output_column = 0;
            }
            if (residual_bits < 5)
                break;
        }
    }
    return size;
}
