//
// uncia - a big cat
// Copyright (C) 2010 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_OUTPUT_FILTER_BASE32HEX_H
#define LIBUNCIA_OUTPUT_FILTER_BASE32HEX_H

#include <libuncia/output/filter.h>

namespace libuncia
{
    /**
      * The output_filter_base32hex class is used to represent an output filter
      * which encodes using the MIME base-32-hex encoding (see RFC 4648).
      */
    class output_filter_base32hex:
        public output_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~output_filter_base32hex();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     where to send the output of this filter.
          */
        output_filter_base32hex(const pointer &deeper);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     where to send the output of this filter.
          */
        static pointer create(const pointer &deeper);

    protected:
        // See base class for documentation.
        ssize_t write(const void *data, size_t size);

    private:
        /**
          * The residual_value instance variable is used to remember the
          * as-yet-unwritten bits of the value.
          */
        unsigned int residual_value;

        /**
          * The residual_bits instance variable is used to remember the
          * number of bits held by the residual_value instance variable.
          */
        int residual_bits;

        /**
          * The output_column instance variable is used to remember what
          * column the output is up to.
          */
        int output_column;

        /**
          * The default constructor.  Do not use.
          */
        output_filter_base32hex();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        output_filter_base32hex(const output_filter_base32hex &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        output_filter_base32hex &operator=(const output_filter_base32hex &rhs);
    };
}

#endif // LIBUNCIA_OUTPUT_FILTER_BASE32HEX_H
