//
// uncia - a big cat
// Copyright (C) 2010 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libuncia/output/filter/primos.h>


libuncia::output_filter_primos::~output_filter_primos()
{
}


libuncia::output_filter_primos::output_filter_primos(const pointer &a_deeper) :
    output_filter(a_deeper),
    address(0),
    column(0),
    spaces(0)
{
}


libuncia::output::pointer
libuncia::output_filter_primos::create(const pointer &a_deeper)
{
    return pointer(new output_filter_primos(a_deeper));
}


ssize_t
libuncia::output_filter_primos::write(const void *data, size_t size)
{
    const char *start = (const char *)data;
    const char *end = start + size;
    const char *cp = start;
    while (cp < end)
    {
        unsigned char c = *cp++;
        switch (c)
        {
        case ' ':
            ++spaces;
            ++column;
            break;

        case '\t':
            while (column & 7)
            {
                ++spaces;
                ++column;
            }
            break;

        case '\n':
            column = 0;
            deeper_putc('\n' ^ 0x80);
            ++address;
            column = 0;
            if (address & 1)
            {
                deeper_putc('\0');
                ++address;
            }
            break;

        default:
            while (spaces >= 255)
            {
                deeper_putc(021);
                deeper_putc(255);
                address += 2;
                spaces -= 255;
            }
            switch (spaces)
            {
            case 0:
                break;

            case 2:
                deeper_putc(' ');
                ++address;
                // Fall through...

            case 1:
                deeper_putc(' ');
                ++address;
                break;

            default:
                deeper_putc(021);
                ++address;
                deeper_putc(spaces);
                ++address;
                break;
            }
            spaces = 0;
            deeper_putc(c ^ 0x80);
            ++address;
            ++column;
            break;
        }
    }
    return size;
}
