//
// uncia - a big cat
// Copyright (C) 2008, 2009 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <ctime>
#include <cstring>

#include <libuncia/output/filter/prefix.h>
#include <libuncia/output/filter/timestamp.h>


libuncia::output_filter_timestamp::~output_filter_timestamp()
{
}


libuncia::output_filter_timestamp::output_filter_timestamp(
        const pointer &a_deeper, const std::string &a_strftime_format) :
    output_filter(a_deeper),
    strftime_format(a_strftime_format),
    start_of_line(true)
{
}


libuncia::output_filter_timestamp::pointer
libuncia::output_filter_timestamp::create(const pointer &a_deeper)
{
    return create(a_deeper, "%Y-%m-%d %H:%M:%S");
}


libuncia::output_filter_timestamp::pointer
libuncia::output_filter_timestamp::create(const pointer &a_deeper,
    const std::string &a_strftime_format)
{
    if (a_strftime_format.empty())
        return a_deeper;
    if (!strchr(a_strftime_format.c_str(), '%'))
        return output_filter_prefix::create(a_deeper, a_strftime_format);
    return pointer(new output_filter_timestamp(a_deeper, a_strftime_format));
}


ssize_t
libuncia::output_filter_timestamp::write(const void *data, size_t size)
{
    const char *cp = (const char *)data;
    const char *ep = cp + size;
    while (cp < ep)
    {
        if (start_of_line)
        {
            time_t now = 0;
            time(&now);
            struct tm *tmp = localtime(&now);
            assert(tmp);
            char buf[100];
            strftime(buf, sizeof(buf), strftime_format.c_str(), tmp);
            deeper_puts(buf);
            deeper_puts(": ");
            start_of_line = false;
        }
        char c = *cp++;
        deeper_putc(c);
        if (c == '\n')
            start_of_line = true;
    }
    return size;
}
