//
// uncia - a big cat
// Copyright (C) 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <cstring>

#include <libuncia/output/filter/vms.h>


libuncia::output_filter_vms::~output_filter_vms()
{
    if (bufpos > 0)
        emit_one_line();
    deeper_putc(0xFF);
    deeper_putc(0xFF);
    address += 2;
    while (address & 0x01FF)
    {
        deeper_putc('\0');
        ++address;
    }
}


libuncia::output_filter_vms::output_filter_vms(const pointer &a_deeper) :
    output_filter(a_deeper),
    bufpos(0),
    address(0)
{
}


libuncia::output::pointer
libuncia::output_filter_vms::create(const pointer &a_deeper)
{
    return pointer(new output_filter_vms(a_deeper));
}


void
libuncia::output_filter_vms::emit_one_line(void)
{
    deeper_putc(bufpos);
    deeper_putc(bufpos >> 8);
    address += 2;
    deeper_write(buffer, bufpos);
    address += bufpos;
    if (address & 1)
    {
        deeper_putc('\0');
        ++address;
    }
    bufpos = 0;
}


ssize_t
libuncia::output_filter_vms::write(const void *data, size_t size)
{
    const char *start = (const char *)data;
    const char *end = start + size;
    const char *cp = start;
    while (cp < end)
    {
        unsigned char c = *cp++;
        if (c == '\n')
        {
            emit_one_line();
        }
        else
        {
            buffer[bufpos++] = c;

            // Watch out for long lines (but don't act prematurely, or
            // you will introduce a spurious blank line).
            if (bufpos > max_line_len)
            {
                size_t extra = bufpos - max_line_len;
                bufpos = max_line_len;
                emit_one_line();
                memmove(buffer, buffer + max_line_len, extra);
                bufpos = extra;
            }
        }
    }
    return (cp - start);
}


// vim: set ts=8 sw=4 et :
