//
// uncia - a big cat
// Copyright (C) 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_OUTPUT_FILTER_VMS_H
#define LIBUNCIA_OUTPUT_FILTER_VMS_H

#include <libuncia/ostream.h>
#include <libuncia/output/filter.h>

namespace libuncia
{
    /**
      * The output_filter_vms class is used to represent an output
      * filter which turns a normalised text file into a VMS formatted
      * text file.
      */
    class output_filter_vms:
        public output_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~output_filter_vms();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     where to send the output of this filter.
          */
        output_filter_vms(const pointer &deeper);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     where to send the output of this filter.
          */
        static pointer create(const pointer &deeper);

    protected:
        // See base class for documentation.
        ssize_t write(const void *data, size_t size);

    private:
        enum { max_line_len = 65534 };
        char buffer[max_line_len + 2];
        size_t bufpos;
        unsigned long address;

        void emit_one_line(void);

        /**
          * The default constructor.  Do not use.
          */
        output_filter_vms();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        output_filter_vms(const output_filter_vms &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        output_filter_vms &operator=(const output_filter_vms &rhs);
    };
}

// vim: set ts=8 sw=4 et :
#endif // LIBUNCIA_OUTPUT_FILTER_VMS_H
