//
// uncia - a big cat
// Copyright (C) 2008, 2010 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libuncia/config.h>
#include <libexplain/program_name.h>
#include <syslog.h>

#include <libuncia/output/syslog.h>


libuncia::output::pointer libuncia::output_syslog::singleton;


libuncia::output_syslog::~output_syslog()
{
    closelog();
}


libuncia::output_syslog::output_syslog(int a_facility, int a_level,
        int option):
    facility(a_facility < 0 ? LOG_USER : a_facility),
    level(a_level < 0 ? LOG_ERR : a_level)
{
    if (option < 0)
        option = 0;
    const char *ident = explain_program_name_get();
    openlog(ident, option, facility);
}


libuncia::output::pointer
libuncia::output_syslog::create(int a_facility, int a_level, int option)
{
    if (!singleton)
        singleton = pointer(new output_syslog(a_facility, a_level, option));
    return singleton;
}


ssize_t
libuncia::output_syslog::write(const void *data, size_t size)
{
    const char *start = (const char *)data;
    const char *end = start + size;
    const char *cp = start;
    while (cp < end)
    {
        char c = *cp++;
        line += c;
        if (c == '\n')
        {
            int priority = facility | level;
            syslog(priority, "%s", line.c_str());
            line.clear();
        }
    }
    return size;
}


int
libuncia::output_syslog::sync()
{
    if (!line.empty())
        write("\n", 1);
    return 0;
}


std::string
libuncia::output_syslog::filename_get()
    const
{
    return "syslog";
}
